/* -*- Mode: java; tab-width: 8; c-basic-offset: 4 -*-
 * Copyright  1997, 1998 Netscape Communications Corporation, All Rights Reserved.
 */

package com.netscape.javascript.CORBA;

import com.netscape.javascript.*;
import org.omg.CORBA.InterfaceDefPackage.*;
import org.omg.CORBA.DefinitionKind;
import org.omg.CORBA.Contained;
import org.omg.CORBA.ModuleDef;
import org.omg.CORBA.ModuleDefHelper;
import org.omg.CORBA.InterfaceDefHelper;
import org.omg.CORBA.StructDefHelper;
import org.omg.CORBA.EnumDefHelper;
import org.omg.CORBA.ExceptionDefHelper;
import org.omg.CORBA.Request;

/**
 * This class roots the CORBA Modules space, providing a hierarchy for finding type information
 * such as interfaces and structure definitions.  It also provides a reflection for CORBA
 * ModuleDefs found below the root space.
 *
 * @author Mike Shaver
 */

public class Module extends ScriptableObject {

    public static Object init(Scriptable scope) {

	Module proto = new Module();
	proto.setParentScope(scope);
	proto.setPrototype(ScriptableObject.getObjectPrototype(scope));
	scope.put("Modules", scope, proto);

        return proto;
    }

    public Scriptable create() { return new Module(); }
    public String getClassName() { return "Module"; }
    public Module(ModuleDef module) { this.module = module; }
    public Module() { this.module = null; }

    public Object getDefaultValue(Class hint) {
    	if (hint == null ||
	        hint == String.class)
	        return "[object CORBAModule" + (module == null ? "" : " " +
					    module.absolute_name()) + "]";
    	return this;
    }

    /**
     * XXX more wrap() nonsense.
     */
    public static Scriptable wrap(Contained component) {
	switch(component.def_kind().value()) {
            case DefinitionKind._dk_Module:
	        return new Module(ModuleDefHelper.narrow(component));
       	    case DefinitionKind._dk_Interface:
	        return new Interface(InterfaceDefHelper.narrow(component));
            case DefinitionKind._dk_Exception:
	        return new Struct(ExceptionDefHelper.narrow (component).type ());
            case DefinitionKind._dk_Struct:
	        return new Struct(StructDefHelper.narrow (component).type ());
            case DefinitionKind._dk_Enum:
	        return new Enum  (EnumDefHelper.narrow (component).type ());
    	    default:
	        throw new JavaScriptException("CORBA module doesn't know how to reflect DefKind == " + 
					  component.def_kind().value());
	}
    }

    public boolean has(String id, int index, Scriptable start) {
        if ("valueOf".equals(id) || "toString".equals(id))
	    return false;
    	return true;
    }

    public void set(String id, int index, Scriptable start, Object value) {
	// nope
    }

    public Object get (String id, Scriptable start)
    {
	Contained component;
	if (module == null)	// root Module, so use the repository top
	    component = CORBAObject.rep.lookup(id);
    	else
	    component = module.lookup(id);

	if (component == null)
	    return Scriptable.NOT_FOUND;

	Scriptable newValue;
	newValue = wrap (component);

	super.put (id, this, newValue);

	return newValue;
    }

    ModuleDef module = null;
}
